@@ -51,6 +51,7 @@ typedef enum {
 
 typedef enum {
 	GSD_PRIO_BUILTIN,            /* Output is builtin, applies mainly to system-integrated devices */
+	GSD_PRIO_MATCH_SIZE,	     /* Size from input device and output match */
 	GSD_PRIO_EDID_MATCH_FULL,    /* Full EDID model match, eg. "Cintiq 12WX" */
 	GSD_PRIO_EDID_MATCH_PARTIAL, /* Partial EDID model match, eg. "Cintiq" */
 	GSD_PRIO_EDID_MATCH_VENDOR,  /* EDID vendor match, eg. "WAC" for Wacom */
@@ -120,6 +121,9 @@ enum {
 	N_SIGNALS
 };
 
+static GnomeRROutput * input_info_find_size_match (GsdInputInfo  *input,
+                                                   GnomeRRScreen *rr_screen);
+
 static guint signals[N_SIGNALS] = { 0 };
 
 G_DEFINE_TYPE (GsdDeviceMapper, gsd_device_mapper, G_TYPE_OBJECT)
@@ -334,8 +338,14 @@ input_info_guess_candidates (GsdInputInfo  *input,
 	gboolean found = FALSE;
 	const gchar *name;
 	gchar **split;
+	gint i;
 
 	name = gdk_device_get_name (input->device);
+
+	if (input->capabilities & GSD_INPUT_IS_SCREEN_INTEGRATED) {
+		outputs[GSD_PRIO_MATCH_SIZE] = input_info_find_size_match (input, input->mapper->rr_screen);
+	}
+
 	split = g_strsplit (name, " ", -1);
 
 	/* On Wacom devices that are integrated on a not-in-system screen (eg. Cintiqs),
@@ -351,14 +361,13 @@ input_info_guess_candidates (GsdInputInfo  *input,
 			{ "WAC", split[1], NULL },
 			{ "WAC", NULL, NULL }
 		};
-		gint i;
 
 		for (i = 0; i < G_N_ELEMENTS (edids); i++) {
 			/* i + 1 matches the desired priority, we skip GSD_PRIO_BUILTIN here */
-			outputs[i + 1] =
+			outputs[i + GSD_PRIO_EDID_MATCH_FULL] =
 				find_output_by_edid (input->mapper->rr_screen,
 						     edids[i]);
-			found |= outputs[i + 1] != NULL;
+			found |= outputs[i + GSD_PRIO_EDID_MATCH_FULL] != NULL;
 		}
 
 		g_free (product);
@@ -496,7 +505,8 @@ input_info_get_output (GsdInputInfo *input)
 	if (input->guessed_output)
 		return input->guessed_output;
 
-	if (g_hash_table_size (input->mapper->output_devices) == 1) {
+	if (input->mapper->output_devices &&
+	    g_hash_table_size (input->mapper->output_devices) == 1) {
 		GsdOutputInfo *output;
 		GHashTableIter iter;
 
@@ -593,6 +603,78 @@ monitor_for_output (GnomeRROutput *output)
 	return gdk_screen_get_monitor_at_point (screen, x, y);
 }
 
+static gboolean
+output_get_dimensions (GnomeRROutput *output,
+		       guint         *width,
+		       guint         *height)
+{
+	GdkScreen *screen = gdk_screen_get_default ();
+	gint monitor_num;
+
+	monitor_num = monitor_for_output (output);
+
+	if (monitor_num < 0)
+		return FALSE;
+
+	*width = gdk_screen_get_monitor_width_mm (screen, monitor_num);
+	*height = gdk_screen_get_monitor_height_mm (screen, monitor_num);
+	return TRUE;
+}
+
+static GnomeRROutput *
+input_info_find_size_match (GsdInputInfo  *input,
+			    GnomeRRScreen *rr_screen)
+{
+	guint i, input_width, input_height, output_width, output_height;
+	gdouble min_width_diff, min_height_diff;
+	GnomeRROutput **outputs, *match = NULL;
+
+	g_return_val_if_fail (rr_screen != NULL, NULL);
+
+	if (!xdevice_get_dimensions (gdk_x11_device_get_id (input->device),
+				     &input_width, &input_height))
+		return NULL;
+
+	/* Restrict the matches to be below a narrow percentage */
+	min_width_diff = min_height_diff = 0.05;
+
+	g_debug ("Input device '%s' has %dx%d mm",
+		 gdk_device_get_name (input->device), input_width, input_height);
+
+	outputs = gnome_rr_screen_list_outputs (rr_screen);
+
+	for (i = 0; outputs[i] != NULL; i++) {
+		gdouble width_diff, height_diff;
+
+		if (!output_get_dimensions (outputs[i], &output_width, &output_height))
+			continue;
+
+		width_diff = ABS (1 - ((gdouble) output_width / input_width));
+		height_diff = ABS (1 - ((gdouble) output_height / input_height));
+
+		g_debug ("Output '%s' has size %dx%d mm, deviation from "
+			 "input device size: %.2f width, %.2f height ",
+			 gnome_rr_output_get_name (outputs[i]),
+			 output_width, output_height, width_diff, height_diff);
+
+		if (width_diff <= min_width_diff && height_diff <= min_height_diff) {
+			match = outputs[i];
+			min_width_diff = width_diff;
+			min_height_diff = height_diff;
+		}
+	}
+
+	if (match) {
+		g_debug ("Output '%s' is considered a best size match (%.2f / %.2f)",
+			 gnome_rr_output_get_name (match),
+			 min_width_diff, min_height_diff);
+	} else {
+		g_debug ("No input/output size match was found\n");
+	}
+
+	return match;
+}
+
 static void
 input_info_get_matrix (GsdInputInfo *input,
 		       float	     matrix[NUM_ELEMS_MATRIX])
@@ -780,6 +862,8 @@ input_info_update_capabilities_from_tool_type (GsdInputInfo *info)
 		info->capabilities |= GSD_INPUT_IS_ERASER;
 	else if (g_str_equal (tool_type, "PAD"))
 		info->capabilities |= GSD_INPUT_IS_PAD;
+	else
+		return FALSE;
 
 	return TRUE;
 }
@@ -876,7 +960,8 @@ input_info_new (GdkDevice	*device,
 						     info);
 
 		/* Assign output from config */
-		rr_output = settings_get_display (settings, mapper);
+		if (mapper->rr_screen)
+			rr_output = settings_get_display (settings, mapper);
 	}
 
 	input_info_update_capabilities (info);
