@@ -244,6 +244,8 @@ should_purge_file (GFile        *file,
                                   G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
                                   cancellable,
                                   NULL);
+        if (!info)
+                return FALSE;
 
         date = g_file_info_get_deletion_date (info);
         if (date == NULL) {
@@ -269,18 +271,7 @@ out:
         return should_purge;
 }
 
-typedef struct {
-        gint ref_count;
-        GFile           *file;
-        GCancellable    *cancellable;
-        GDateTime       *old;
-        gboolean         dry_run;
-        gboolean         trash;
-        gchar           *name;
-        gint             depth;
-} DeleteData;
-
-static DeleteData *
+DeleteData *
 delete_data_new (GFile        *file,
                  GCancellable *cancellable,
                  GDateTime    *old,
@@ -310,7 +301,7 @@ delete_data_ref (DeleteData *data)
   return data;
 }
 
-static void
+void
 delete_data_unref (DeleteData *data)
 {
         data->ref_count -= 1;
@@ -325,8 +316,6 @@ delete_data_unref (DeleteData *data)
         g_free (data);
 }
 
-static void delete_recursively_by_age (DeleteData *data);
-
 static void
 delete_batch (GObject      *source,
               GAsyncResult *res,
@@ -427,6 +416,45 @@ delete_subdir (GObject      *source,
 }
 
 static void
+delete_subdir_check_symlink (GObject      *source,
+                             GAsyncResult *res,
+                             gpointer      user_data)
+{
+        GFile *file = G_FILE (source);
+        DeleteData *data = user_data;
+        GFileInfo *info;
+        GFileType type;
+
+        info = g_file_query_info_finish (file, res, NULL);
+        if (!info) {
+                delete_data_unref (data);
+                return;
+        }
+
+        type = g_file_info_get_file_type (info);
+        g_object_unref (info);
+
+        if (type == G_FILE_TYPE_DIRECTORY) {
+                g_file_enumerate_children_async (data->file,
+                                                 G_FILE_ATTRIBUTE_STANDARD_NAME ","
+                                                 G_FILE_ATTRIBUTE_STANDARD_TYPE,
+                                                 G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                                 0,
+                                                 data->cancellable,
+                                                 delete_subdir,
+                                                 delete_data_ref (data));
+        } else if (type == G_FILE_TYPE_SYMBOLIC_LINK) {
+                if (should_purge_file (data->file, data->cancellable, data->old)) {
+                        g_debug ("Purging %s leaf node", data->name);
+                        if (!data->dry_run) {
+                                g_file_delete (data->file, data->cancellable, NULL);
+                        }
+                }
+        }
+        delete_data_unref (data);
+}
+
+void
 delete_recursively_by_age (DeleteData *data)
 {
         if (data->trash && (data->depth == 1) &&
@@ -435,14 +463,13 @@ delete_recursively_by_age (DeleteData *data)
                 return;
         }
 
-        g_file_enumerate_children_async (data->file,
-                                         G_FILE_ATTRIBUTE_STANDARD_NAME ","
-                                         G_FILE_ATTRIBUTE_STANDARD_TYPE,
-                                         G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
-                                         0,
-                                         data->cancellable,
-                                         delete_subdir,
-                                         delete_data_ref (data));
+        g_file_query_info_async (data->file,
+                                 G_FILE_ATTRIBUTE_STANDARD_TYPE,
+                                 G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                 0,
+                                 data->cancellable,
+                                 delete_subdir_check_symlink,
+                                 delete_data_ref (data));
 }
 
 void
@@ -946,6 +973,7 @@ ldsm_mounts_changed (GObject  *monitor,
                 g_source_remove (ldsm_timeout_id);
         ldsm_timeout_id = g_timeout_add_seconds (CHECK_EVERY_X_SECONDS,
                                                  ldsm_check_all_mounts, NULL);
+        g_source_set_name_by_id (ldsm_timeout_id, "[gnome-settings-daemon] ldsm_check_all_mounts");
 }
 
 static gboolean
@@ -1027,8 +1055,10 @@ gsd_ldsm_setup (gboolean check_now)
 
         ldsm_timeout_id = g_timeout_add_seconds (CHECK_EVERY_X_SECONDS,
                                                  ldsm_check_all_mounts, NULL);
+        g_source_set_name_by_id (ldsm_timeout_id, "[gnome-settings-daemon] ldsm_check_all_mounts");
 
         purge_trash_id = g_timeout_add_seconds (3600, ldsm_purge_trash_and_temp, NULL);
+        g_source_set_name_by_id (purge_trash_id, "[gnome-settings-daemon] ldsm_purge_trash_and_temp");
 }
 
 void
