@@ -468,9 +468,12 @@ grab_accelerators_complete (GObject      *object,
                 }
         }
 
-        if (retry)
-                g_timeout_add_seconds (SHELL_GRABBER_RETRY_INTERVAL,
-                                       retry_grabs, manager);
+        if (retry) {
+                guint id;
+                id = g_timeout_add_seconds (SHELL_GRABBER_RETRY_INTERVAL,
+                                            retry_grabs, manager);
+                g_source_set_name_by_id (id, "[gnome-settings-daemon] retry_grabs");
+        }
 }
 
 static void
@@ -579,8 +582,14 @@ gsettings_changed_cb (GSettings           *settings,
         if (g_str_equal (settings_key, "max-screencast-length"))
 		return;
 
+	/* Handled by the core */
+        if (g_str_equal (settings_key, "active"))
+                return;
 
         /* Find the key that was modified */
+        if (manager->priv->keys == NULL)
+                return;
+
         for (i = 0; i < manager->priv->keys->len; i++) {
                 MediaKey *key;
 
@@ -959,7 +968,7 @@ do_execute_desktop_or_desktop (GsdMediaKeysManager *manager,
         GDesktopAppInfo *app_info;
 
         app_info = g_desktop_app_info_new (desktop);
-        if (app_info == NULL)
+        if (app_info == NULL && alt_desktop != NULL)
                 app_info = g_desktop_app_info_new (alt_desktop);
 
         if (app_info != NULL) {
@@ -1985,10 +1994,12 @@ screencast_start (GsdMediaKeysManager *manager)
 
         max_length = g_settings_get_int (manager->priv->settings, "max-screencast-length");
 
-        if (max_length > 0)
+        if (max_length > 0) {
                 manager->priv->screencast_timeout_id = g_timeout_add_seconds (max_length,
                                                                               screencast_timeout,
                                                                               manager);
+                g_source_set_name_by_id (manager->priv->screencast_timeout_id, "[gnome-settings-daemon] screencast_timeout");
+        }
         manager->priv->screencast_recording = TRUE;
 }
 
@@ -2091,6 +2102,9 @@ do_action (GsdMediaKeysManager *manager,
         case CALCULATOR_KEY:
                 do_execute_desktop_or_desktop (manager, "gcalctool.desktop", "gnome-calculator.desktop", timestamp);
                 break;
+        case CONTROL_CENTER_KEY:
+                do_execute_desktop_or_desktop (manager, "gnome-control-center.desktop", NULL, timestamp);
+                break;
         case PLAY_KEY:
                 return do_multimedia_player_action (manager, "Play");
         case PAUSE_KEY:
@@ -2404,6 +2418,7 @@ gsd_media_keys_manager_start (GsdMediaKeysManager *manager,
 #endif
 
         manager->priv->start_idle_id = g_idle_add ((GSourceFunc) start_media_keys_idle_cb, manager);
+        g_source_set_name_by_id (manager->priv->start_idle_id, "[gnome-settings-daemon] start_media_keys_idle_cb");
 
         register_manager (manager_object);
 
