@@ -21,12 +21,15 @@
 
 #include <locale.h>
 #include <glib.h>
+#include <gio/gio.h>
+#include <gio/gdesktopappinfo.h>
+#include <glib/gstdio.h>
 
+#ifdef HAVE_NETWORK_MANAGER
 #include <nm-client.h>
 #include <nm-device.h>
 #include <nm-remote-settings.h>
-#include <gio/gdesktopappinfo.h>
-#include <glib/gstdio.h>
+#endif /* HAVE_NETWORK_MANAGER */
 
 #include "gnome-settings-plugin.h"
 #include "gnome-settings-profile.h"
@@ -49,12 +52,15 @@ struct GsdSharingManagerPrivate
         GDBusConnection         *connection;
 
         GCancellable            *cancellable;
+#ifdef HAVE_NETWORK_MANAGER
         NMClient                *client;
         NMRemoteSettings        *remote_settings;
+#endif /* HAVE_NETWORK_MANAGER */
 
         GHashTable              *services;
 
         char                    *current_network;
+        char                    *current_network_name;
         char                    *carrier_type;
         GsdSharingStatus         sharing_status;
 };
@@ -67,6 +73,7 @@ static const gchar introspection_xml[] =
 "  <interface name='org.gnome.SettingsDaemon.Sharing'>"
 "    <annotation name='org.freedesktop.DBus.GLib.CSymbol' value='gsd_sharing_manager'/>"
 "    <property name='CurrentNetwork' type='s' access='read'/>"
+"    <property name='CurrentNetworkName' type='s' access='read'/>"
 "    <property name='CarrierType' type='s' access='read'/>"
 "    <property name='SharingStatus' type='u' access='read'/>"
 "    <method name='EnableService'>"
@@ -78,7 +85,7 @@ static const gchar introspection_xml[] =
 "    </method>"
 "    <method name='ListNetworks'>"
 "      <arg name='service-name' direction='in' type='s'/>"
-"      <arg name='networks' direction='out' type='a{ss}'/>"
+"      <arg name='networks' direction='out' type='a(sss)'/>"
 "    </method>"
 "  </interface>"
 "</node>";
@@ -120,11 +127,11 @@ gsd_sharing_manager_start_service (GsdSharingManager *manager,
         }
 
         exec = g_app_info_get_commandline (G_APP_INFO (app));
-        g_object_unref (app);
 
         if (!g_shell_parse_argv (exec, NULL, &argvp, &error)) {
                 g_warning ("Could not parse command-line '%s': %s", exec, error->message);
                 g_error_free (error);
+                g_object_unref (app);
                 return;
         }
 
@@ -139,31 +146,30 @@ gsd_sharing_manager_start_service (GsdSharingManager *manager,
         }
 
         g_strfreev (argvp);
+        g_object_unref (app);
 }
 
-static void
-gsd_sharing_manager_start_services (GsdSharingManager *manager)
+#ifdef HAVE_NETWORK_MANAGER
+static gboolean
+service_is_enabled_on_current_connection (GsdSharingManager *manager,
+                                          ServiceInfo       *service)
 {
-        GList *services, *l;
-
-        services = g_hash_table_get_values (manager->priv->services);
-
-        for (l = services; l != NULL; l = l->next) {
-                ServiceInfo *service = l->data;
-                char **connections;
-                guint j;
-
-                connections = g_settings_get_strv (service->settings, "enabled-connections");
-                for (j = 0; connections[j] != NULL; j++) {
-                        if (g_strcmp0 (connections[j], manager->priv->current_network) == 0) {
-                                gsd_sharing_manager_start_service (manager, service);
-                                break;
-                        }
+        char **connections;
+        int j;
+        gboolean ret;
+        connections = g_settings_get_strv (service->settings, "enabled-connections");
+        ret = FALSE;
+        for (j = 0; connections[j] != NULL; j++) {
+                if (g_strcmp0 (connections[j], manager->priv->current_network) == 0) {
+                        ret = TRUE;
+                        break;
                 }
-                g_strfreev (connections);
         }
-        g_list_free (services);
+
+        g_strfreev (connections);
+        return ret;
 }
+#endif /* HAVE_NETWORK_MANAGER */
 
 static void
 gsd_sharing_manager_stop_service (GsdSharingManager *manager,
@@ -182,7 +188,7 @@ gsd_sharing_manager_stop_service (GsdSharingManager *manager,
 }
 
 static void
-gsd_sharing_manager_stop_services (GsdSharingManager *manager)
+gsd_sharing_manager_sync_services (GsdSharingManager *manager)
 {
         GList *services, *l;
 
@@ -190,11 +196,23 @@ gsd_sharing_manager_stop_services (GsdSharingManager *manager)
 
         for (l = services; l != NULL; l = l->next) {
                 ServiceInfo *service = l->data;
-                gsd_sharing_manager_stop_service (manager, service);
+                gboolean should_be_started = FALSE;
+
+                if (manager->priv->sharing_status == GSD_SHARING_STATUS_AVAILABLE &&
+                    service_is_enabled_on_current_connection (manager, service))
+                        should_be_started = TRUE;
+
+                if (service->started != should_be_started) {
+                        if (service->started)
+                                gsd_sharing_manager_stop_service (manager, service);
+                        else
+                                gsd_sharing_manager_start_service (manager, service);
+                }
         }
         g_list_free (services);
 }
 
+#ifdef HAVE_NETWORK_MANAGER
 static void
 properties_changed (GsdSharingManager *manager)
 {
@@ -209,6 +227,8 @@ properties_changed (GsdSharingManager *manager)
 
         g_variant_builder_add (&props_builder, "{sv}", "CurrentNetwork",
                                g_variant_new_string (manager->priv->current_network));
+        g_variant_builder_add (&props_builder, "{sv}", "CurrentNetworkName",
+                               g_variant_new_string (manager->priv->current_network_name));
         g_variant_builder_add (&props_builder, "{sv}", "CarrierType",
                                g_variant_new_string (manager->priv->carrier_type));
         g_variant_builder_add (&props_builder, "{sv}", "SharingStatus",
@@ -235,6 +255,15 @@ get_connections_for_service (GsdSharingManager *manager,
         service = g_hash_table_lookup (manager->priv->services, service_name);
         return g_settings_get_strv (service->settings, "enabled-connections");
 }
+#else
+static char **
+get_connections_for_service (GsdSharingManager *manager,
+                             const char        *service_name)
+{
+        const char * const * connections [] = { NULL };
+        return g_strdupv ((char **) connections);
+}
+#endif /* HAVE_NETWORK_MANAGER */
 
 static gboolean
 check_service (GsdSharingManager  *manager,
@@ -324,9 +353,11 @@ gsd_sharing_manager_disable_service (GsdSharingManager  *manager,
         return TRUE;
 }
 
+#ifdef HAVE_NETWORK_MANAGER
 static const char *
-get_type_for_connection_id (GsdSharingManager *manager,
-                            const char        *id)
+get_type_and_name_for_connection_uuid (GsdSharingManager *manager,
+                                       const char        *uuid,
+                                       const char       **name)
 {
         NMRemoteConnection *conn;
         const char *type;
@@ -334,24 +365,35 @@ get_type_for_connection_id (GsdSharingManager *manager,
         if (!manager->priv->remote_settings)
                 return NULL;
 
-        conn = nm_remote_settings_get_connection_by_id (manager->priv->remote_settings, id);
+        conn = nm_remote_settings_get_connection_by_uuid (manager->priv->remote_settings, uuid);
         if (!conn)
                 return NULL;
         type = nm_connection_get_connection_type (NM_CONNECTION (conn));
+        *name = nm_connection_get_id (NM_CONNECTION (conn));
 
         return type;
 }
+#else
+static const char *
+get_type_and_name_for_connection_uuid (GsdSharingManager *manager,
+                                       const char        *id,
+                                       const char       **name)
+{
+        return NULL;
+}
+#endif /* HAVE_NETWORK_MANAGER */
 
+#ifdef HAVE_NETWORK_MANAGER
 static gboolean
 connection_is_low_security (GsdSharingManager *manager,
-                            const char        *id)
+                            const char        *uuid)
 {
         NMRemoteConnection *conn;
 
         if (!manager->priv->remote_settings)
                 return TRUE;
 
-        conn = nm_remote_settings_get_connection_by_id (manager->priv->remote_settings, id);
+        conn = nm_remote_settings_get_connection_by_uuid (manager->priv->remote_settings, uuid);
         if (!conn)
                 return TRUE;
 
@@ -359,6 +401,7 @@ connection_is_low_security (GsdSharingManager *manager,
          * XXX: Also do this for WEP networks? */
         return (nm_connection_get_setting_wireless_security (NM_CONNECTION (conn)) == NULL);
 }
+#endif /* HAVE_NETWORK_MANAGER */
 
 static GVariant *
 gsd_sharing_manager_list_networks (GsdSharingManager  *manager,
@@ -372,24 +415,26 @@ gsd_sharing_manager_list_networks (GsdSharingManager  *manager,
         if (!check_service (manager, service_name, error))
                 return NULL;
 
+#ifdef HAVE_NETWORK_MANAGER
         if (!manager->priv->remote_settings) {
                 g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_FAILED, "Not ready yet");
                 return NULL;
         }
+#endif /* HAVE_NETWORK_MANAGER */
 
         connections = get_connections_for_service (manager, service_name);
 
-        g_variant_builder_init (&builder, G_VARIANT_TYPE ("(a{ss})"));
-        g_variant_builder_open (&builder, G_VARIANT_TYPE ("a{ss}"));
+        g_variant_builder_init (&builder, G_VARIANT_TYPE ("(a(sss))"));
+        g_variant_builder_open (&builder, G_VARIANT_TYPE ("a(sss)"));
 
         for (i = 0; connections[i] != NULL; i++) {
-                const char *type;
+                const char *type, *name;
 
-                type = get_type_for_connection_id (manager, connections[i]);
+                type = get_type_and_name_for_connection_uuid (manager, connections[i], &name);
                 if (!type)
                         continue;
 
-                g_variant_builder_add (&builder, "{ss}", connections[i], type);
+                g_variant_builder_add (&builder, "(sss)", connections[i], name, type);
         }
         g_strfreev (connections);
 
@@ -411,15 +456,17 @@ handle_get_property (GDBusConnection *connection,
 
         /* Check session pointer as a proxy for whether the manager is in the
            start or stop state */
-        if (manager->priv->connection == NULL ||
-            manager->priv->client == NULL) {
+        if (manager->priv->connection == NULL)
                 return NULL;
-        }
 
         if (g_strcmp0 (property_name, "CurrentNetwork") == 0) {
                 return g_variant_new_string (manager->priv->current_network);
         }
 
+        if (g_strcmp0 (property_name, "CurrentNetworkName") == 0) {
+                return g_variant_new_string (manager->priv->current_network_name);
+        }
+
         if (g_strcmp0 (property_name, "CarrierType") == 0) {
                 return g_variant_new_string (manager->priv->carrier_type);
         }
@@ -447,10 +494,8 @@ handle_method_call (GDBusConnection       *connection,
 
         /* Check session pointer as a proxy for whether the manager is in the
            start or stop state */
-        if (manager->priv->connection == NULL ||
-            manager->priv->client == NULL) {
+        if (manager->priv->connection == NULL)
                 return;
-        }
 
         if (g_strcmp0 (method_name, "EnableService") == 0) {
                 const char *service;
@@ -525,6 +570,7 @@ on_bus_gotten (GObject               *source_object,
                                                                NULL);
 }
 
+#ifdef HAVE_NETWORK_MANAGER
 static void
 primary_connection_changed (GObject    *gobject,
                             GParamSpec *pspec,
@@ -536,19 +582,24 @@ primary_connection_changed (GObject    *gobject,
         a_con = nm_client_get_primary_connection (manager->priv->client);
 
         g_clear_pointer (&manager->priv->current_network, g_free);
+        g_clear_pointer (&manager->priv->current_network_name, g_free);
         g_clear_pointer (&manager->priv->carrier_type, g_free);
 
         if (a_con) {
-                manager->priv->current_network = g_strdup (nm_active_connection_get_id (a_con));
+                manager->priv->current_network = g_strdup (nm_active_connection_get_uuid (a_con));
+                manager->priv->current_network_name = g_strdup (nm_active_connection_get_id (a_con));
                 manager->priv->carrier_type = g_strdup (nm_active_connection_get_connection_type (a_con));
         } else {
                 manager->priv->current_network = g_strdup ("");
+                manager->priv->current_network_name = g_strdup ("");
                 manager->priv->carrier_type = g_strdup ("");
         }
 
         if (!a_con) {
                 manager->priv->sharing_status = GSD_SHARING_STATUS_OFFLINE;
-        } else if (g_str_equal (manager->priv->carrier_type, "bluetooth")) {
+        } else if (g_str_equal (manager->priv->carrier_type, "bluetooth") ||
+                   g_str_equal (manager->priv->carrier_type, "gsm") ||
+                   g_str_equal (manager->priv->carrier_type, "cdma")) {
                 manager->priv->sharing_status = GSD_SHARING_STATUS_DISABLED_MOBILE_BROADBAND;
         } else if (g_str_equal (manager->priv->carrier_type, "802-11-wireless")) {
                 if (connection_is_low_security (manager, manager->priv->current_network))
@@ -560,15 +611,12 @@ primary_connection_changed (GObject    *gobject,
         }
 
         g_debug ("current network: %s", manager->priv->current_network);
+        g_debug ("current network name: %s", manager->priv->current_network_name);
         g_debug ("conn type: %s", manager->priv->carrier_type);
         g_debug ("status: %d", manager->priv->sharing_status);
 
         properties_changed (manager);
-
-        if (manager->priv->sharing_status == GSD_SHARING_STATUS_AVAILABLE)
-                gsd_sharing_manager_start_services (manager);
-        else
-                gsd_sharing_manager_stop_services (manager);
+        gsd_sharing_manager_sync_services (manager);
 }
 
 static void
@@ -614,6 +662,7 @@ remote_settings_ready_cb (GObject      *source_object,
 
         manager->priv->remote_settings = remote_settings;
 }
+#endif /* HAVE_NETWORK_MANAGER */
 
 #define RYGEL_BUS_NAME "org.gnome.Rygel1"
 #define RYGEL_OBJECT_PATH "/org/gnome/Rygel1"
@@ -627,9 +676,10 @@ gsd_sharing_manager_disable_rygel (void)
 
 	path = g_build_filename (g_get_user_config_dir (), "autostart",
 				 "rygel.desktop", NULL);
-	if (g_file_test (path, G_FILE_TEST_IS_SYMLINK | G_FILE_TEST_IS_REGULAR))
-		g_unlink (path);
-	g_free (path);
+	if (!g_file_test (path, G_FILE_TEST_IS_SYMLINK | G_FILE_TEST_IS_REGULAR))
+                goto out;
+
+        g_unlink (path);
 
 	connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
 	if (connection) {
@@ -638,6 +688,9 @@ gsd_sharing_manager_disable_rygel (void)
 					NULL, NULL, NULL);
 	}
 	g_object_unref (connection);
+
+ out:
+        g_free (path);
 }
 
 gboolean
@@ -653,8 +706,10 @@ gsd_sharing_manager_start (GsdSharingManager *manager,
         gsd_sharing_manager_disable_rygel ();
 
         manager->priv->cancellable = g_cancellable_new ();
+#ifdef HAVE_NETWORK_MANAGER
         nm_client_new_async (manager->priv->cancellable, nm_client_ready, manager);
         nm_remote_settings_new_async (NULL, manager->priv->cancellable, remote_settings_ready_cb, manager);
+#endif /* HAVE_NETWORK_MANAGER */
 
         /* Start process of owning a D-Bus name */
         g_bus_get (G_BUS_TYPE_SESSION,
@@ -671,14 +726,18 @@ gsd_sharing_manager_stop (GsdSharingManager *manager)
 {
         g_debug ("Stopping sharing manager");
 
-        gsd_sharing_manager_stop_services (manager);
+        manager->priv->sharing_status = GSD_SHARING_STATUS_OFFLINE;
+        gsd_sharing_manager_sync_services (manager);
 
         if (manager->priv->cancellable) {
                 g_cancellable_cancel (manager->priv->cancellable);
                 g_clear_object (&manager->priv->cancellable);
         }
+
+#ifdef HAVE_NETWORK_MANAGER
         g_clear_object (&manager->priv->client);
         g_clear_object (&manager->priv->remote_settings);
+#endif /* HAVE_NETWORK_MANAGER */
 
         if (manager->priv->name_id != 0) {
                 g_bus_unown_name (manager->priv->name_id);
@@ -689,6 +748,7 @@ gsd_sharing_manager_stop (GsdSharingManager *manager)
         g_clear_object (&manager->priv->connection);
 
         g_clear_pointer (&manager->priv->current_network, g_free);
+        g_clear_pointer (&manager->priv->current_network_name, g_free);
         g_clear_pointer (&manager->priv->carrier_type, g_free);
 }
 
@@ -722,6 +782,7 @@ gsd_sharing_manager_init (GsdSharingManager *manager)
 
         /* Default state */
         manager->priv->current_network = g_strdup ("");
+        manager->priv->current_network_name = g_strdup ("");
         manager->priv->carrier_type = g_strdup ("");
         manager->priv->sharing_status = GSD_SHARING_STATUS_OFFLINE;
 
